@@ -158,12 +158,12 @@ class MetricScheduler(MetricProcessBase):
     MAX_OVERLAP = 0.3
     GROUP_ID = "gnocchi-scheduler"
     SYNC_RATE = 30
-    TASKS_PER_WORKER = 16
     BLOCK_SIZE = 4
 
     def __init__(self, worker_id, conf, queue):
         super(MetricScheduler, self).__init__(
             worker_id, conf, conf.metricd.metric_processing_delay)
+        self.TASKS_PER_WORKER = conf.metricd.tasks_per_worker
         self.queue = queue
         self.previously_scheduled_metrics = set()
         self.workers = conf.metricd.workers
@@ -178,7 +178,14 @@ class MetricScheduler(MetricProcessBase):
         get_members_req = self._coord.get_members(self.GROUP_ID)
         try:
             members = sorted(get_members_req.get())
-            self.block_index = members.index(self._my_id)
+            try:
+                self.block_index = members.index(self._my_id)
+            except ValueError:
+                # NOTE(sileht): we didn't stop and wait the periodic
+                # because set_block() is the periodic method, so just
+                # close the coordinator and exit
+                self.close_coordinator()
+                raise SystemExit(6)
             reqs = list(self._coord.get_member_capabilities(self.GROUP_ID, m)
                         for m in members)
             for req in reqs:
@@ -267,6 +274,9 @@ class MetricScheduler(MetricProcessBase):
         if self.periodic:
             self.periodic.stop()
             self.periodic.wait()
+        self.close_coordinator()
+
+    def close_coordinator(self):
         self._coord.leave_group(self.GROUP_ID)
         self._coord.stop()
 
