@@ -49,8 +49,7 @@ class Resource(object):
                 and self.revision == other.revision
                 and self.revision_start == other.revision_start
                 and self.revision_end == other.revision_end
-                and self.created_by_user_id == other.created_by_user_id
-                and self.created_by_project_id == other.created_by_project_id
+                and self.creator == other.creator
                 and self.user_id == other.user_id
                 and self.project_id == other.project_id
                 and self.started_at == other.started_at
@@ -115,6 +114,16 @@ class NoSuchArchivePolicy(IndexerException):
         self.archive_policy = archive_policy
 
 
+class UnsupportedArchivePolicyChange(IndexerException):
+    """Error raised when modifying archive policy if not supported."""
+    def __init__(self, archive_policy, message):
+        super(UnsupportedArchivePolicyChange, self).__init__(
+            "Archive policy %s does not support change: %s" %
+            (archive_policy, message))
+        self.archive_policy = archive_policy
+        self.message = message
+
+
 class ArchivePolicyInUse(IndexerException):
     """Error raised when an archive policy is still being used."""
     def __init__(self, archive_policy):
@@ -131,6 +140,17 @@ class ResourceTypeInUse(IndexerException):
         self.resource_type = resource_type
 
 
+class UnexpectedResourceTypeState(IndexerException):
+    """Error raised when an resource type state is not expected."""
+    def __init__(self, resource_type, expected_state, state):
+        super(UnexpectedResourceTypeState, self).__init__(
+            "Resource type %s state is %s (expected: %s)" % (
+                resource_type, state, expected_state))
+        self.resource_type = resource_type
+        self.expected_state = expected_state
+        self.state = state
+
+
 class NoSuchArchivePolicyRule(IndexerException):
     """Error raised when an archive policy rule does not exist."""
     def __init__(self, archive_policy_rule):
@@ -177,8 +197,8 @@ class ResourceAttributeError(IndexerException, AttributeError):
     """Error raised when an attribute does not exist for a resource type."""
     def __init__(self, resource, attribute):
         super(ResourceAttributeError, self).__init__(
-            "Resource %s has no %s attribute" % (resource, attribute))
-        self.resource = resource,
+            "Resource type %s has no %s attribute" % (resource, attribute))
+        self.resource = resource
         self.attribute = attribute
 
 
@@ -284,6 +304,10 @@ class IndexerDriver(object):
     def get_archive_policy(name):
         raise exceptions.NotImplementedError
 
+    @staticmethod
+    def update_archive_policy(name, ap_items):
+        raise exceptions.NotImplementedError
+
     @staticmethod
     def delete_archive_policy(name):
         raise exceptions.NotImplementedError
@@ -305,13 +329,14 @@ class IndexerDriver(object):
         raise exceptions.NotImplementedError
 
     @staticmethod
-    def create_metric(id, created_by_user_id, created_by_project_id,
-                      archive_policy_name, name=None, resource_id=None):
+    def create_metric(id, creator,
+                      archive_policy_name, name=None, unit=None,
+                      resource_id=None):
         raise exceptions.NotImplementedError
 
     @staticmethod
     def list_metrics(names=None, ids=None, details=False, status='active',
-                     **kwargs):
+                     limit=None, marker=None, sorts=None, **kwargs):
         raise exceptions.NotImplementedError
 
     @staticmethod
@@ -319,7 +344,8 @@ class IndexerDriver(object):
         raise exceptions.NotImplementedError
 
     @staticmethod
-    def create_resource(resource_type, id, user_id, project_id,
+    def create_resource(resource_type, id, creator,
+                        user_id=None, project_id=None,
                         started_at=None, ended_at=None, metrics=None,
                         **kwargs):
         raise exceptions.NotImplementedError
@@ -336,6 +362,11 @@ class IndexerDriver(object):
     def delete_resource(uuid):
         raise exceptions.NotImplementedError
 
+    @staticmethod
+    def delete_resources(resource_type='generic',
+                         attribute_filter=None):
+        raise exceptions.NotImplementedError
+
     @staticmethod
     def delete_metric(id):
         raise exceptions.NotImplementedError
