@@ -1,9 +1,8 @@
 # -*- encoding: utf-8 -*-
 #
+# Copyright © 2016 Red Hat, Inc.
 # Copyright © 2014-2015 eNovance
 #
-# Authors: Julien Danjou <julien@danjou.info>
-#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -26,6 +25,7 @@ from oslo_utils import timeutils
 from oslo_utils import units
 import six
 import sqlalchemy
+from sqlalchemy.dialects import mysql
 from sqlalchemy.ext import declarative
 from sqlalchemy import types
 import sqlalchemy_utils
@@ -44,7 +44,11 @@ COMMON_TABLES_ARGS = {'mysql_charset': "utf8",
 
 
 class PreciseTimestamp(types.TypeDecorator):
-    """Represents a timestamp precise to the microsecond."""
+    """Represents a timestamp precise to the microsecond.
+
+    Deprecated in favor of TimestampUTC.
+    Still used in alembic migrations.
+    """
 
     impl = sqlalchemy.DateTime
 
@@ -102,6 +106,25 @@ class PreciseTimestamp(types.TypeDecorator):
                 tzinfo=iso8601.iso8601.UTC)
 
 
+class TimestampUTC(types.TypeDecorator):
+    """Represents a timestamp precise to the microsecond."""
+
+    impl = sqlalchemy.DateTime
+
+    def load_dialect_impl(self, dialect):
+        if dialect.name == 'mysql':
+            return dialect.type_descriptor(mysql.DATETIME(fsp=6))
+        return self.impl
+
+    def process_bind_param(self, value, dialect):
+        if value is not None:
+            return timeutils.normalize_time(value)
+
+    def process_result_value(self, value, dialect):
+        if value is not None:
+            return value.replace(tzinfo=iso8601.iso8601.UTC)
+
+
 class GnocchiBase(models.ModelBase):
     __table_args__ = (
         COMMON_TABLES_ARGS,
@@ -148,19 +171,17 @@ class Metric(Base, GnocchiBase, storage.Metric):
         sqlalchemy.ForeignKey(
             'archive_policy.name',
             ondelete="RESTRICT",
-            name="fk_metric_archive_policy_name_archive_policy_name"),
+            name="fk_metric_ap_name_ap_name"),
         nullable=False)
     archive_policy = sqlalchemy.orm.relationship(ArchivePolicy, lazy="joined")
-    created_by_user_id = sqlalchemy.Column(
-        sqlalchemy.String(255))
-    created_by_project_id = sqlalchemy.Column(
-        sqlalchemy.String(255))
+    creator = sqlalchemy.Column(sqlalchemy.String(255))
     resource_id = sqlalchemy.Column(
         sqlalchemy_utils.UUIDType(),
         sqlalchemy.ForeignKey('resource.id',
                               ondelete="SET NULL",
                               name="fk_metric_resource_id_resource_id"))
     name = sqlalchemy.Column(sqlalchemy.String(255))
+    unit = sqlalchemy.Column(sqlalchemy.String(31))
     status = sqlalchemy.Column(sqlalchemy.Enum('active', 'delete',
                                                name="metric_status_enum"),
                                nullable=False,
@@ -169,9 +190,9 @@ class Metric(Base, GnocchiBase, storage.Metric):
     def jsonify(self):
         d = {
             "id": self.id,
-            "created_by_user_id": self.created_by_user_id,
-            "created_by_project_id": self.created_by_project_id,
+            "creator": self.creator,
             "name": self.name,
+            "unit": self.unit,
         }
         unloaded = sqlalchemy.inspect(self).unloaded
         if 'resource' in unloaded:
@@ -182,6 +203,14 @@ class Metric(Base, GnocchiBase, storage.Metric):
             d['archive_policy_name'] = self.archive_policy_name
         else:
             d['archive_policy'] = self.archive_policy
+
+        if self.creator is None:
+            d['created_by_user_id'] = d['created_by_project_id'] = None
+        else:
+            d['created_by_user_id'], _, d['created_by_project_id'] = (
+                self.creator.partition(":")
+            )
+
         return d
 
     def __eq__(self, other):
@@ -192,9 +221,9 @@ class Metric(Base, GnocchiBase, storage.Metric):
         return ((isinstance(other, Metric)
                  and self.id == other.id
                  and self.archive_policy_name == other.archive_policy_name
-                 and self.created_by_user_id == other.created_by_user_id
-                 and self.created_by_project_id == other.created_by_project_id
+                 and self.creator == other.creator
                  and self.name == other.name
+                 and self.unit == other.unit
                  and self.resource_id == other.resource_id)
                 or (storage.Metric.__eq__(self, other)))
 
@@ -213,6 +242,7 @@ def get_legacy_resource_types():
             attributes)
         resource_types.append(ResourceType(name=name,
                                            tablename=tablename,
+                                           state="creating",
                                            attributes=attrs))
     return resource_types
 
@@ -238,8 +268,23 @@ class ResourceType(Base, GnocchiBase, resource_type.ResourceType):
 
     name = sqlalchemy.Column(sqlalchemy.String(255), primary_key=True,
                              nullable=False)
-    tablename = sqlalchemy.Column(sqlalchemy.String(18), nullable=False)
+    tablename = sqlalchemy.Column(sqlalchemy.String(35), nullable=False)
     attributes = sqlalchemy.Column(ResourceTypeAttributes)
+    state = sqlalchemy.Column(sqlalchemy.Enum("active", "creating",
+                                              "creation_error", "deleting",
+                                              "deletion_error", "updating",
+                                              "updating_error",
+                                              name="resource_type_state_enum"),
+                              nullable=False,
+                              server_default="creating")
+    updated_at = sqlalchemy.Column(TimestampUTC, nullable=False,
+                                   # NOTE(jd): We would like to use
+                                   # sqlalchemy.func.now, but we can't
+                                   # because the type of PreciseTimestamp in
+                                   # MySQL is not a Timestamp, so it would
+                                   # not store a timestamp but a date as an
+                                   # integer.
+                                   default=lambda: utils.utcnow())
 
     def to_baseclass(self):
         cols = {}
@@ -256,6 +301,14 @@ class ResourceJsonifier(indexer.Resource):
         if 'metrics' not in sqlalchemy.inspect(self).unloaded:
             d['metrics'] = dict((m.name, six.text_type(m.id))
                                 for m in self.metrics)
+
+        if self.creator is None:
+            d['created_by_user_id'] = d['created_by_project_id'] = None
+        else:
+            d['created_by_user_id'], _, d['created_by_project_id'] = (
+                self.creator.partition(":")
+            )
+
         return d
 
 
@@ -276,24 +329,16 @@ class ResourceMixin(ResourceJsonifier):
                                   cls.__tablename__),
             nullable=False)
 
-    created_by_user_id = sqlalchemy.Column(
-        sqlalchemy.String(255))
-    created_by_project_id = sqlalchemy.Column(
-        sqlalchemy.String(255))
-    started_at = sqlalchemy.Column(PreciseTimestamp, nullable=False,
-                                   # NOTE(jd): We would like to use
-                                   # sqlalchemy.func.now, but we can't
-                                   # because the type of PreciseTimestamp in
-                                   # MySQL is not a Timestamp, so it would
-                                   # not store a timestamp but a date as an
-                                   # integer.
+    creator = sqlalchemy.Column(sqlalchemy.String(255))
+    started_at = sqlalchemy.Column(TimestampUTC, nullable=False,
                                    default=lambda: utils.utcnow())
-    revision_start = sqlalchemy.Column(PreciseTimestamp, nullable=False,
+    revision_start = sqlalchemy.Column(TimestampUTC, nullable=False,
                                        default=lambda: utils.utcnow())
-    ended_at = sqlalchemy.Column(PreciseTimestamp)
+    ended_at = sqlalchemy.Column(TimestampUTC)
     user_id = sqlalchemy.Column(sqlalchemy.String(255))
     project_id = sqlalchemy.Column(sqlalchemy.String(255))
-    original_resource_id = sqlalchemy.Column(sqlalchemy.String(255))
+    original_resource_id = sqlalchemy.Column(sqlalchemy.String(255),
+                                             nullable=False)
 
 
 class Resource(ResourceMixin, Base, GnocchiBase):
@@ -327,9 +372,9 @@ class ResourceHistory(ResourceMixin, Base, GnocchiBase):
                            sqlalchemy.ForeignKey(
                                'resource.id',
                                ondelete="CASCADE",
-                               name="fk_resource_history_id_resource_id"),
+                               name="fk_rh_id_resource_id"),
                            nullable=False)
-    revision_end = sqlalchemy.Column(PreciseTimestamp, nullable=False,
+    revision_end = sqlalchemy.Column(TimestampUTC, nullable=False,
                                      default=lambda: utils.utcnow())
     metrics = sqlalchemy.orm.relationship(
         Metric, primaryjoin="Metric.resource_id == ResourceHistory.id",
@@ -350,12 +395,15 @@ class ResourceExtMixin(object):
 
     @declarative.declared_attr
     def id(cls):
+        tablename_compact = cls.__tablename__
+        if tablename_compact.endswith("_history"):
+            tablename_compact = tablename_compact[:-6]
         return sqlalchemy.Column(
             sqlalchemy_utils.UUIDType(),
             sqlalchemy.ForeignKey(
                 'resource.id',
                 ondelete="CASCADE",
-                name="fk_%s_id_resource_id" % cls.__tablename__,
+                name="fk_%s_id_resource_id" % tablename_compact,
                 # NOTE(sileht): We use to ensure that postgresql
                 # does not use AccessExclusiveLock on destination table
                 use_alter=True),
@@ -370,13 +418,16 @@ class ResourceHistoryExtMixin(object):
 
     @declarative.declared_attr
     def revision(cls):
+        tablename_compact = cls.__tablename__
+        if tablename_compact.endswith("_history"):
+            tablename_compact = tablename_compact[:-6]
         return sqlalchemy.Column(
             sqlalchemy.Integer,
             sqlalchemy.ForeignKey(
                 'resource_history.revision',
                 ondelete="CASCADE",
-                name="fk_%s_revision_resource_history_revision"
-                % cls.__tablename__,
+                name="fk_%s_revision_rh_revision"
+                % tablename_compact,
                 # NOTE(sileht): We use to ensure that postgresql
                 # does not use AccessExclusiveLock on destination table
                 use_alter=True),
@@ -403,7 +454,6 @@ class ArchivePolicyRule(Base, GnocchiBase):
         sqlalchemy.ForeignKey(
             'archive_policy.name',
             ondelete="RESTRICT",
-            name="fk_archive_policy_rule_"
-            "archive_policy_name_archive_policy_name"),
+            name="fk_apr_ap_name_ap_name"),
         nullable=False)
     metric_pattern = sqlalchemy.Column(sqlalchemy.String(255), nullable=False)
