@@ -18,7 +18,6 @@ import collections
 import datetime
 import itertools
 import operator
-import struct
 import uuid
 
 from concurrent import futures
@@ -26,9 +25,7 @@ import iso8601
 import msgpack
 from oslo_config import cfg
 from oslo_log import log
-from oslo_serialization import msgpackutils
 from oslo_utils import timeutils
-import pandas
 import six
 import six.moves
 from tooz import coordination
@@ -61,11 +58,10 @@ class CorruptionError(ValueError):
 
 
 class CarbonaraBasedStorage(storage.StorageDriver):
-    MEASURE_PREFIX = "measure"
     UPGRADE_BATCH_SIZE = 1000
 
-    def __init__(self, conf):
-        super(CarbonaraBasedStorage, self).__init__(conf)
+    def __init__(self, conf, incoming):
+        super(CarbonaraBasedStorage, self).__init__(conf, incoming)
         self.aggregation_workers_number = conf.aggregation_workers_number
         if self.aggregation_workers_number == 1:
             # NOTE(jd) Avoid using futures at all if we don't want any threads.
@@ -106,8 +102,8 @@ class CarbonaraBasedStorage(storage.StorageDriver):
             )
             LOG.debug(
                 "Retrieve unaggregated measures "
-                "for %s in %.2fs"
-                % (metric.id, sw.elapsed()))
+                "for %s in %.2fs",
+                metric.id, sw.elapsed())
         try:
             return carbonara.BoundTimeSerie.unserialize(
                 raw_measures, block_size, back_window)
@@ -145,7 +141,7 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                          else attrs[-1] == 'v%s' % v)
 
     def get_measures(self, metric, from_timestamp=None, to_timestamp=None,
-                     aggregation='mean', granularity=None):
+                     aggregation='mean', granularity=None, resample=None):
         super(CarbonaraBasedStorage, self).get_measures(
             metric, from_timestamp, to_timestamp, aggregation)
         if granularity is None:
@@ -155,9 +151,13 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                   from_timestamp, to_timestamp)
                  for ap in reversed(metric.archive_policy.definition)))
         else:
-            agg_timeseries = [self._get_measures_timeserie(
+            agg_timeseries = self._get_measures_timeserie(
                 metric, aggregation, granularity,
-                from_timestamp, to_timestamp)]
+                from_timestamp, to_timestamp)
+            if resample:
+                agg_timeseries = agg_timeseries.resample(resample)
+            agg_timeseries = [agg_timeseries]
+
         return [(timestamp.replace(tzinfo=iso8601.iso8601.UTC), r, v)
                 for ts in agg_timeseries
                 for timestamp, r, v in ts.fetch(from_timestamp, to_timestamp)]
@@ -171,8 +171,8 @@ class CarbonaraBasedStorage(storage.StorageDriver):
         except carbonara.InvalidData:
             LOG.error("Data corruption detected for %s "
                       "aggregated `%s' timeserie, granularity `%s' "
-                      "around time `%s', ignoring."
-                      % (metric.id, aggregation, granularity, key))
+                      "around time `%s', ignoring.",
+                      metric.id, aggregation, granularity, key)
 
     def _get_measures_timeserie(self, metric,
                                 aggregation, granularity,
@@ -230,7 +230,7 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                                oldest_mutable_timestamp):
         # NOTE(jd) We write the full split only if the driver works that way
         # (self.WRITE_FULL) or if the oldest_mutable_timestamp is out of range.
-        write_full = self.WRITE_FULL or next(key) < oldest_mutable_timestamp
+        write_full = self.WRITE_FULL or next(key) <= oldest_mutable_timestamp
         key_as_str = str(key)
         if write_full:
             try:
@@ -321,6 +321,9 @@ class CarbonaraBasedStorage(storage.StorageDriver):
             if previous_oldest_mutable_key != oldest_mutable_key:
                 for key in existing_keys:
                     if previous_oldest_mutable_key <= key < oldest_mutable_key:
+                        LOG.debug(
+                            "Compressing previous split %s (%s) for metric %s",
+                            key, aggregation, metric)
                         # NOTE(jd) Rewrite it entirely for fun (and later for
                         # compression). For that, we just pass None as split.
                         self._store_timeserie_split(
@@ -331,39 +334,19 @@ class CarbonaraBasedStorage(storage.StorageDriver):
 
         for key, split in ts.split():
             if key >= oldest_key_to_keep:
+                LOG.debug(
+                    "Storing split %s (%s) for metric %s",
+                    key, aggregation, metric)
                 self._store_timeserie_split(
                     metric, key, split, aggregation, archive_policy_def,
                     oldest_mutable_timestamp)
 
-    def add_measures(self, metric, measures):
-        measures = list(measures)
-        data = struct.pack(
-            "<" + self._MEASURE_SERIAL_FORMAT * len(measures),
-            *list(
-                itertools.chain(
-                    # NOTE(jd) int(10e8) to avoid rounding errors
-                    *((int(utils.datetime_to_unix(timestamp) * int(10e8)),
-                       value)
-                      for timestamp, value in measures))))
-        self._store_new_measures(metric, data)
-
-    @staticmethod
-    def _store_new_measures(metric, data):
-        raise NotImplementedError
-
     @staticmethod
     def _delete_metric(metric):
         raise NotImplementedError
 
-    @staticmethod
-    def list_metric_with_measures_to_process(size, part, full=False):
-        raise NotImplementedError
-
-    @staticmethod
-    def _pending_measures_to_process_count(metric_id):
-        raise NotImplementedError
-
     def delete_metric(self, metric, sync=False):
+        LOG.debug("Deleting metric %s", metric)
         with self._lock(metric.id)(blocking=sync):
             # If the metric has never been upgraded, we need to delete this
             # here too
@@ -374,49 +357,24 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                                 aggregation, granularity, version=3):
         raise NotImplementedError
 
-    _MEASURE_SERIAL_FORMAT = "Qd"
-    _MEASURE_SERIAL_LEN = struct.calcsize(_MEASURE_SERIAL_FORMAT)
-
-    def _unserialize_measures(self, measure_id, data):
-        nb_measures = len(data) // self._MEASURE_SERIAL_LEN
-        try:
-            measures = struct.unpack(
-                "<" + self._MEASURE_SERIAL_FORMAT * nb_measures, data)
-        except struct.error:
-            # This either a corruption, either a v2 measures
-            try:
-                return msgpackutils.loads(data)
-            except ValueError:
-                LOG.error(
-                    "Unable to decode measure %s, possible data corruption",
-                    measure_id)
-                raise
-        return six.moves.zip(
-            pandas.to_datetime(measures[::2], unit='ns'),
-            itertools.islice(measures, 1, len(measures), 2))
-
-    def measures_report(self, details=True):
-        metrics, measures, full_details = self._build_report(details)
-        report = {'summary': {'metrics': metrics, 'measures': measures}}
-        if full_details is not None:
-            report['details'] = full_details
-        return report
-
     def _check_for_metric_upgrade(self, metric):
+        # FIXME(gordc): this is only required for v2.x to v3.x storage upgrade.
+        # we should make storage version easily detectable rather than
+        # checking each metric individually
         lock = self._lock(metric.id)
         with lock:
             try:
                 old_unaggregated = self._get_unaggregated_timeserie_and_unserialize_v2(  # noqa
                     metric)
             except (storage.MetricDoesNotExist, CorruptionError) as e:
-                # NOTE(jd) This case is not really possible – you can't
-                # have archives with splits and no unaggregated
-                # timeserie…
-                LOG.error(
-                    "Unable to find unaggregated timeserie for "
-                    "metric %s, unable to upgrade data: %s",
+                # This case can happen if v3.0 to v3.x or if no measures
+                # pushed. skip the rest of upgrade on metric.
+                LOG.debug(
+                    "Unable to find v2 unaggregated timeserie for "
+                    "metric %s, no data to upgrade: %s",
                     metric.id, e)
                 return
+
             unaggregated = carbonara.BoundTimeSerie(
                 ts=old_unaggregated.ts,
                 block_size=metric.archive_policy.max_block_size,
@@ -431,8 +389,8 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                     metric.archive_policy.aggregation_methods,
                     metric.archive_policy.definition):
                 LOG.debug(
-                    "Checking if the metric %s needs migration for %s"
-                    % (metric, agg_method))
+                    "Checking if the metric %s needs migration for %s",
+                    metric, agg_method)
 
                 try:
                     all_keys = self._list_split_keys_for_metric(
@@ -441,7 +399,7 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                     # Just try the next metric, this one has no measures
                     break
                 else:
-                    LOG.info("Migrating metric %s to new format" % metric)
+                    LOG.info("Migrating metric %s to new format", metric)
                     timeseries = filter(
                         lambda x: x is not None,
                         self._map_in_thread(
@@ -463,7 +421,7 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                             metric, key, agg_method,
                             d.granularity, version=None)
             self._delete_unaggregated_timeserie(metric, version=None)
-            LOG.info("Migrated metric %s to new format" % metric)
+            LOG.info("Migrated metric %s to new format", metric)
 
     def upgrade(self, index):
         marker = None
@@ -476,7 +434,8 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                 break
             marker = metrics[-1][0].id
 
-    def process_new_measures(self, indexer, metrics_to_process, sync=False):
+    def process_new_measures(self, indexer, metrics_to_process,
+                             sync=False):
         metrics = indexer.list_metrics(ids=metrics_to_process)
         # This build the list of deleted metrics, i.e. the metrics we have
         # measures to process for but that are not in the indexer anymore.
@@ -488,129 +447,126 @@ class CarbonaraBasedStorage(storage.StorageDriver):
             # measurement files under its feet is not nice!
             try:
                 with self._lock(metric_id)(blocking=sync):
-                    self._delete_unprocessed_measures_for_metric_id(metric_id)
+                    self.incoming.delete_unprocessed_measures_for_metric_id(
+                        metric_id)
             except coordination.LockAcquireFailed:
                 LOG.debug("Cannot acquire lock for metric %s, postponing "
-                          "unprocessed measures deletion" % metric_id)
+                          "unprocessed measures deletion", metric_id)
+
         for metric in metrics:
             lock = self._lock(metric.id)
-            agg_methods = list(metric.archive_policy.aggregation_methods)
             # Do not block if we cannot acquire the lock, that means some other
             # worker is doing the job. We'll just ignore this metric and may
             # get back later to it if needed.
-            if lock.acquire(blocking=sync):
-                try:
-                    locksw = timeutils.StopWatch().start()
-                    LOG.debug("Processing measures for %s" % metric)
-                    with self._process_measure_for_metric(metric) as measures:
-                        # NOTE(mnaser): The metric could have been handled by
-                        #               another worker, ignore if no measures.
-                        if len(measures) == 0:
-                            LOG.debug("Skipping %s (already processed)"
-                                      % metric)
-                            continue
+            if not lock.acquire(blocking=sync):
+                continue
+            try:
+                locksw = timeutils.StopWatch().start()
+                LOG.debug("Processing measures for %s", metric)
+                with self.incoming.process_measure_for_metric(metric) \
+                        as measures:
+                    self._compute_and_store_timeseries(metric, measures)
+                LOG.debug("Metric %s locked during %.2f seconds",
+                          metric.id, locksw.elapsed())
+            except Exception:
+                LOG.debug("Metric %s locked during %.2f seconds",
+                          metric.id, locksw.elapsed())
+                if sync:
+                    raise
+                LOG.error("Error processing new measures", exc_info=True)
+            finally:
+                lock.release()
 
-                        measures = sorted(measures, key=operator.itemgetter(0))
+    def _compute_and_store_timeseries(self, metric, measures):
+        # NOTE(mnaser): The metric could have been handled by
+        #               another worker, ignore if no measures.
+        if len(measures) == 0:
+            LOG.debug("Skipping %s (already processed)", metric)
+            return
 
-                        block_size = metric.archive_policy.max_block_size
-                        try:
-                            ts = self._get_unaggregated_timeserie_and_unserialize(  # noqa
-                                metric,
-                                block_size=block_size,
-                                back_window=metric.archive_policy.back_window)
-                        except storage.MetricDoesNotExist:
-                            try:
-                                self._create_metric(metric)
-                            except storage.MetricAlreadyExists:
-                                # Created in the mean time, do not worry
-                                pass
-                            ts = None
-                        except CorruptionError as e:
-                            LOG.error(e)
-                            ts = None
+        measures = sorted(measures, key=operator.itemgetter(0))
 
-                        if ts is None:
-                            # This is the first time we treat measures for this
-                            # metric, or data are corrupted, create a new one
-                            ts = carbonara.BoundTimeSerie(
-                                block_size=block_size,
-                                back_window=metric.archive_policy.back_window)
-                            current_first_block_timestamp = None
-                        else:
-                            current_first_block_timestamp = (
-                                ts.first_block_timestamp()
-                            )
+        agg_methods = list(metric.archive_policy.aggregation_methods)
+        block_size = metric.archive_policy.max_block_size
+        back_window = metric.archive_policy.back_window
+        definition = metric.archive_policy.definition
 
-                        # NOTE(jd) This is Python where you need such
-                        # hack to pass a variable around a closure,
-                        # sorry.
-                        computed_points = {"number": 0}
+        try:
+            ts = self._get_unaggregated_timeserie_and_unserialize(
+                metric, block_size=block_size, back_window=back_window)
+        except storage.MetricDoesNotExist:
+            try:
+                self._create_metric(metric)
+            except storage.MetricAlreadyExists:
+                # Created in the mean time, do not worry
+                pass
+            ts = None
+        except CorruptionError as e:
+            LOG.error(e)
+            ts = None
 
-                        def _map_add_measures(bound_timeserie):
-                            # NOTE (gordc): bound_timeserie is entire set of
-                            # unaggregated measures matching largest
-                            # granularity. the following takes only the points
-                            # affected by new measures for specific granularity
-                            tstamp = max(bound_timeserie.first, measures[0][0])
-                            computed_points['number'] = len(bound_timeserie)
-                            for d in metric.archive_policy.definition:
-                                ts = bound_timeserie.group_serie(
-                                    d.granularity, carbonara.round_timestamp(
-                                        tstamp, d.granularity * 10e8))
-                                self._map_in_thread(
-                                    self._add_measures,
-                                    ((aggregation, d, metric, ts,
-                                      current_first_block_timestamp,
-                                      bound_timeserie.first_block_timestamp())
-                                     for aggregation in agg_methods))
+        if ts is None:
+            # This is the first time we treat measures for this
+            # metric, or data are corrupted, create a new one
+            ts = carbonara.BoundTimeSerie(block_size=block_size,
+                                          back_window=back_window)
+            current_first_block_timestamp = None
+        else:
+            current_first_block_timestamp = ts.first_block_timestamp()
 
-                        with timeutils.StopWatch() as sw:
-                            ts.set_values(
-                                measures,
-                                before_truncate_callback=_map_add_measures,
-                                ignore_too_old_timestamps=True)
-                            elapsed = sw.elapsed()
-                            number_of_operations = (
-                                len(agg_methods)
-                                * len(metric.archive_policy.definition)
-                            )
+        # NOTE(jd) This is Python where you need such
+        # hack to pass a variable around a closure,
+        # sorry.
+        computed_points = {"number": 0}
 
-                            if elapsed > 0:
-                                perf = " (%d points/s, %d measures/s)" % (
-                                    ((number_of_operations
-                                      * computed_points['number']) / elapsed),
-                                    ((number_of_operations
-                                      * len(measures)) / elapsed)
-                                )
-                            else:
-                                perf = ""
-                            LOG.debug(
-                                "Computed new metric %s with %d new measures "
-                                "in %.2f seconds%s"
-                                % (metric.id, len(measures), elapsed, perf))
+        def _map_add_measures(bound_timeserie):
+            # NOTE (gordc): bound_timeserie is entire set of
+            # unaggregated measures matching largest
+            # granularity. the following takes only the points
+            # affected by new measures for specific granularity
+            tstamp = max(bound_timeserie.first, measures[0][0])
+            new_first_block_timestamp = bound_timeserie.first_block_timestamp()
+            computed_points['number'] = len(bound_timeserie)
+            for d in definition:
+                ts = bound_timeserie.group_serie(
+                    d.granularity, carbonara.round_timestamp(
+                        tstamp, d.granularity * 10e8))
 
-                        self._store_unaggregated_timeserie(metric,
-                                                           ts.serialize())
+                self._map_in_thread(
+                    self._add_measures,
+                    ((aggregation, d, metric, ts,
+                        current_first_block_timestamp,
+                        new_first_block_timestamp)
+                        for aggregation in agg_methods))
 
-                    LOG.debug("Metric %s locked during %.2f seconds" %
-                              (metric.id, locksw.elapsed()))
-                except Exception:
-                    LOG.debug("Metric %s locked during %.2f seconds" %
-                              (metric.id, locksw.elapsed()))
-                    if sync:
-                        raise
-                    LOG.error("Error processing new measures", exc_info=True)
-                finally:
-                    lock.release()
+        with timeutils.StopWatch() as sw:
+            ts.set_values(measures,
+                          before_truncate_callback=_map_add_measures,
+                          ignore_too_old_timestamps=True)
+
+            elapsed = sw.elapsed()
+            number_of_operations = (len(agg_methods) * len(definition))
+            perf = ""
+            if elapsed > 0:
+                perf = " (%d points/s, %d measures/s)" % (
+                    ((number_of_operations * computed_points['number']) /
+                        elapsed),
+                    ((number_of_operations * len(measures)) / elapsed)
+                )
+            LOG.debug("Computed new metric %s with %d new measures "
+                      "in %.2f seconds%s",
+                      metric.id, len(measures), elapsed, perf)
+
+        self._store_unaggregated_timeserie(metric, ts.serialize())
 
     def get_cross_metric_measures(self, metrics, from_timestamp=None,
                                   to_timestamp=None, aggregation='mean',
-                                  reaggregation=None,
-                                  granularity=None,
-                                  needed_overlap=100.0):
+                                  reaggregation=None, resample=None,
+                                  granularity=None, needed_overlap=100.0,
+                                  fill=None):
         super(CarbonaraBasedStorage, self).get_cross_metric_measures(
             metrics, from_timestamp, to_timestamp,
-            aggregation, reaggregation, granularity, needed_overlap)
+            aggregation, reaggregation, resample, granularity, needed_overlap)
 
         if reaggregation is None:
             reaggregation = aggregation
@@ -634,17 +590,26 @@ class CarbonaraBasedStorage(storage.StorageDriver):
         else:
             granularities_in_common = [granularity]
 
-        tss = self._map_in_thread(self._get_measures_timeserie,
-                                  [(metric, aggregation, g,
-                                    from_timestamp, to_timestamp)
-                                   for metric in metrics
-                                   for g in granularities_in_common])
+        if resample and granularity:
+            tss = self._map_in_thread(self._get_measures_timeserie,
+                                      [(metric, aggregation, granularity,
+                                        from_timestamp, to_timestamp)
+                                       for metric in metrics])
+            for i, ts in enumerate(tss):
+                tss[i] = ts.resample(resample)
+        else:
+            tss = self._map_in_thread(self._get_measures_timeserie,
+                                      [(metric, aggregation, g,
+                                        from_timestamp, to_timestamp)
+                                       for metric in metrics
+                                       for g in granularities_in_common])
+
         try:
             return [(timestamp.replace(tzinfo=iso8601.iso8601.UTC), r, v)
                     for timestamp, r, v
                     in carbonara.AggregatedTimeSerie.aggregated(
                         tss, reaggregation, from_timestamp, to_timestamp,
-                        needed_overlap)]
+                        needed_overlap, fill)]
         except carbonara.UnAggregableTimeseries as e:
             raise storage.MetricUnaggregatable(metrics, e.reason)
 
@@ -660,18 +625,21 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                  for timestamp, g, value in values
                  if predicate(value)]}
 
-    # TODO(jd) Add granularity parameter here and in the REST API
-    # rather than fetching all granularities
     def search_value(self, metrics, query, from_timestamp=None,
-                     to_timestamp=None, aggregation='mean'):
+                     to_timestamp=None, aggregation='mean',
+                     granularity=None):
+        granularity = granularity or []
         predicate = storage.MeasureQuery(query)
+
         results = self._map_in_thread(
             self._find_measure,
             [(metric, aggregation,
-              ap.granularity, predicate,
+              gran, predicate,
               from_timestamp, to_timestamp)
              for metric in metrics
-             for ap in metric.archive_policy.definition])
+             for gran in granularity or
+             (defin.granularity
+              for defin in metric.archive_policy.definition)])
         result = collections.defaultdict(list)
         for r in results:
             for metric, metric_result in six.iteritems(r):
@@ -699,7 +667,8 @@ class CarbonaraBasedStorage(storage.StorageDriver):
     def _unserialize_timeserie_v2(data):
         return carbonara.TimeSerie.from_data(
             *carbonara.TimeSerie._timestamps_and_values_from_dict(
-                msgpack.loads(data, encoding='utf-8')['values']))
+                msgpack.loads(data, encoding='utf-8')['values']),
+            clean=True)
 
     def _get_unaggregated_timeserie_and_unserialize_v2(self, metric):
         """Unserialization method for unaggregated v2 timeseries."""
@@ -707,7 +676,7 @@ class CarbonaraBasedStorage(storage.StorageDriver):
         try:
             return self._unserialize_timeserie_v2(data)
         except ValueError:
-            LOG.error("Data corruption detected for %s ignoring." % metric.id)
+            LOG.error("Data corruption detected for %s ignoring.", metric.id)
 
     def _get_measures_and_unserialize_v2(self, metric, key,
                                          aggregation, granularity):
@@ -719,5 +688,5 @@ class CarbonaraBasedStorage(storage.StorageDriver):
         except ValueError:
             LOG.error("Data corruption detected for %s "
                       "aggregated `%s' timeserie, granularity `%s' "
-                      "around time `%s', ignoring."
-                      % (metric.id, aggregation, granularity, key))
+                      "around time `%s', ignoring.",
+                      metric.id, aggregation, granularity, key)
