@@ -33,6 +33,9 @@ OPTS = [
                help='Ceph username (ie: admin without "client." prefix).'),
     cfg.StrOpt('ceph_secret', help='Ceph key', secret=True),
     cfg.StrOpt('ceph_keyring', help='Ceph keyring path.'),
+    cfg.StrOpt('ceph_timeout',
+               default="30",
+               help='Ceph connection timeout in seconds'),
     cfg.StrOpt('ceph_conffile',
                default='/etc/ceph/ceph.conf',
                help='Ceph configuration file.'),
