@@ -16,6 +16,7 @@
 # under the License.
 from concurrent import futures
 import itertools
+import operator
 import struct
 
 from oslo_log import log
@@ -83,8 +84,20 @@ class CarbonaraBasedStorage(incoming.StorageDriver):
     def _build_report(details):
         raise NotImplementedError
 
+    def list_metric_with_measures_to_process(self, size, part, full=False):
+        metrics = map(operator.itemgetter(0),
+                      # Sort by the number of measures, bigger first (reverse)
+                      sorted(
+                          self._list_metric_with_measures_to_process(),
+                          key=operator.itemgetter(1),
+                          reverse=True))
+        if full:
+            return set(metrics)
+        return set(list(metrics)[size * part:size * (part + 1)])
+
     @staticmethod
-    def list_metric_with_measures_to_process(size, part, full=False):
+    def _list_metric_with_measures_to_process():
+        """Return an ordered list of metrics that needs to be processed."""
         raise NotImplementedError
 
     @staticmethod
