@@ -478,6 +478,32 @@ class TestIndexerDriver(tests_base.TestCase):
         self.assertIn('foo', metric_names)
         self.assertIn('bar', metric_names)
 
+    def test_update_resource_metrics_append_after_delete(self):
+        r1 = uuid.uuid4()
+        m1 = uuid.uuid4()
+        m2 = uuid.uuid4()
+        m3 = uuid.uuid4()
+        creator = str(uuid.uuid4())
+        self.index.create_metric(m1, creator,
+                                 archive_policy_name="low")
+        self.index.create_metric(m2, creator,
+                                 archive_policy_name="low")
+        self.index.create_metric(m3, creator,
+                                 archive_policy_name="low")
+        self.index.create_resource('generic', r1, creator,
+                                   metrics={'foo': m1})
+        rc = self.index.update_resource('generic', r1, metrics={'bar': m2},
+                                        append_metrics=True)
+        self.index.delete_metric(m1)
+        rc = self.index.update_resource('generic', r1, metrics={'foo': m3},
+                                        append_metrics=True)
+        r = self.index.get_resource('generic', r1, with_metrics=True)
+        self.assertEqual(rc, r)
+        metric_names = [m.name for m in rc.metrics]
+        self.assertEqual(2, len(metric_names))
+        self.assertIn('foo', metric_names)
+        self.assertIn('bar', metric_names)
+
     def test_update_resource_metrics_append_fail(self):
         r1 = uuid.uuid4()
         e1 = uuid.uuid4()
@@ -730,6 +756,37 @@ class TestIndexerDriver(tests_base.TestCase):
             attribute_filter={"=": {"project_id": 'bad-project'}})
         self.assertEqual(0, len(resources))
 
+    def test_list_resources_with_no_project(self):
+        r1 = uuid.uuid4()
+        r2 = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
+        creator = user + ":" + project
+        g1 = self.index.create_resource('generic', r1, creator, user, project)
+        g2 = self.index.create_resource('generic', r2, creator, None, None)
+
+        # Get null value
+        resources = self.index.list_resources(
+            'generic',
+            attribute_filter={"and": [
+                {"=": {"creator": creator}},
+                {"!=": {"project_id": project}}
+            ]})
+        self.assertEqual(1, len(resources))
+        self.assertEqual(g2, resources[0])
+
+        # Get null and filled values
+        resources = self.index.list_resources(
+            'generic',
+            attribute_filter={"and": [
+                {"=": {"creator": creator}},
+                {"!=": {"project_id": "foobar"}}
+            ]},
+            sorts=["project_id:asc-nullsfirst"])
+        self.assertEqual(2, len(resources))
+        self.assertEqual(g2, resources[0])
+        self.assertEqual(g1, resources[1])
+
     def test_list_resources_by_duration(self):
         r1 = uuid.uuid4()
         user = str(uuid.uuid4())
@@ -807,6 +864,14 @@ class TestIndexerDriver(tests_base.TestCase):
             resource_type, attribute_filter={"=": {"flavor_id": 1.0}})
         self.assertEqual(0, len(r))
 
+    def test_list_resource_empty_in(self):
+        self.index.create_resource('generic', str(uuid.uuid4()),
+                                   str(uuid.uuid4()), str(uuid.uuid4()))
+        self.assertEqual(
+            [],
+            self.index.list_resources(
+                attribute_filter={"in": {"id": []}}))
+
     def test_list_resource_weird_date(self):
         self.assertRaises(
             indexer.QueryValueError,
