@@ -1,5 +1,6 @@
 # -*- encoding: utf-8 -*-
 #
+# Copyright © 2016 Red Hat, Inc.
 # Copyright © 2015 eNovance
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,11 +14,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-import datetime
 import uuid
 
 import mock
-from oslo_utils import timeutils
 
 from gnocchi import indexer
 from gnocchi import statsd
@@ -35,11 +34,9 @@ class TestStatsd(tests_base.TestCase):
         super(TestStatsd, self).setUp()
 
         self.conf.set_override("resource_id",
-                               uuid.uuid4(), "statsd")
-        self.conf.set_override("user_id",
+                               str(uuid.uuid4()), "statsd")
+        self.conf.set_override("creator",
                                self.STATSD_USER_ID, "statsd")
-        self.conf.set_override("project_id",
-                               self.STATSD_PROJECT_ID, "statsd")
         self.conf.set_override("archive_policy_name",
                                self.STATSD_ARCHIVE_POLICY_NAME, "statsd")
 
@@ -52,11 +49,11 @@ class TestStatsd(tests_base.TestCase):
     def test_flush_empty(self):
         self.server.stats.flush()
 
-    @mock.patch.object(timeutils, 'utcnow')
+    @mock.patch.object(utils, 'utcnow')
     def _test_gauge_or_ms(self, metric_type, utcnow):
         metric_name = "test_gauge_or_ms"
         metric_key = metric_name + "|" + metric_type
-        utcnow.return_value = datetime.datetime(2015, 1, 7, 13, 58, 36)
+        utcnow.return_value = utils.datetime_utc(2015, 1, 7, 13, 58, 36)
         self.server.datagram_received(
             ("%s:1|%s" % (metric_name, metric_type)).encode('ascii'),
             ("127.0.0.1", 12345))
@@ -68,10 +65,8 @@ class TestStatsd(tests_base.TestCase):
 
         metric = r.get_metric(metric_key)
 
-        metrics = self.stats.storage.list_metric_with_measures_to_process(
-            None, None, full=True)
         self.stats.storage.process_background_tasks(
-            self.stats.indexer, metrics, sync=True)
+            self.stats.indexer, [str(metric.id)], sync=True)
 
         measures = self.stats.storage.get_measures(metric)
         self.assertEqual([
@@ -80,7 +75,7 @@ class TestStatsd(tests_base.TestCase):
             (utils.datetime_utc(2015, 1, 7, 13, 58), 60.0, 1.0)
         ], measures)
 
-        utcnow.return_value = datetime.datetime(2015, 1, 7, 13, 59, 37)
+        utcnow.return_value = utils.datetime_utc(2015, 1, 7, 13, 59, 37)
         # This one is going to be ignored
         self.server.datagram_received(
             ("%s:45|%s" % (metric_name, metric_type)).encode('ascii'),
@@ -90,10 +85,8 @@ class TestStatsd(tests_base.TestCase):
             ("127.0.0.1", 12345))
         self.stats.flush()
 
-        metrics = self.stats.storage.list_metric_with_measures_to_process(
-            None, None, full=True)
         self.stats.storage.process_background_tasks(
-            self.stats.indexer, metrics, sync=True)
+            self.stats.indexer, [str(metric.id)], sync=True)
 
         measures = self.stats.storage.get_measures(metric)
         self.assertEqual([
@@ -109,11 +102,11 @@ class TestStatsd(tests_base.TestCase):
     def test_ms(self):
         self._test_gauge_or_ms("ms")
 
-    @mock.patch.object(timeutils, 'utcnow')
+    @mock.patch.object(utils, 'utcnow')
     def test_counter(self, utcnow):
         metric_name = "test_counter"
         metric_key = metric_name + "|c"
-        utcnow.return_value = datetime.datetime(2015, 1, 7, 13, 58, 36)
+        utcnow.return_value = utils.datetime_utc(2015, 1, 7, 13, 58, 36)
         self.server.datagram_received(
             ("%s:1|c" % metric_name).encode('ascii'),
             ("127.0.0.1", 12345))
@@ -125,10 +118,8 @@ class TestStatsd(tests_base.TestCase):
         metric = r.get_metric(metric_key)
         self.assertIsNotNone(metric)
 
-        metrics = self.stats.storage.list_metric_with_measures_to_process(
-            None, None, full=True)
         self.stats.storage.process_background_tasks(
-            self.stats.indexer, metrics, sync=True)
+            self.stats.indexer, [str(metric.id)], sync=True)
 
         measures = self.stats.storage.get_measures(metric)
         self.assertEqual([
@@ -136,7 +127,7 @@ class TestStatsd(tests_base.TestCase):
             (utils.datetime_utc(2015, 1, 7, 13), 3600.0, 1.0),
             (utils.datetime_utc(2015, 1, 7, 13, 58), 60.0, 1.0)], measures)
 
-        utcnow.return_value = datetime.datetime(2015, 1, 7, 13, 59, 37)
+        utcnow.return_value = utils.datetime_utc(2015, 1, 7, 13, 59, 37)
         self.server.datagram_received(
             ("%s:45|c" % metric_name).encode('ascii'),
             ("127.0.0.1", 12345))
@@ -145,10 +136,8 @@ class TestStatsd(tests_base.TestCase):
             ("127.0.0.1", 12345))
         self.stats.flush()
 
-        metrics = self.stats.storage.list_metric_with_measures_to_process(
-            None, None, full=True)
         self.stats.storage.process_background_tasks(
-            self.stats.indexer, metrics, sync=True)
+            self.stats.indexer, [str(metric.id)], sync=True)
 
         measures = self.stats.storage.get_measures(metric)
         self.assertEqual([
